@@ -65,3 +65,16 @@ class OrderType(str, Enum):
 class PeerType(str, Enum):
     NODE = 'node'
     WEBSOCKET = 'ws'
+
+
+class RpcBroadcastRequestType(int, Enum):
+    SYNC = 1
+    ASYNC = 2
+    COMMIT = 3
+
+
+class VoteOption(int, Enum):
+    YES = 1
+    ABSTAIN = 2
+    NO = 3
+    NO_WITH_VETO = 4
