@@ -1,6 +1,6 @@
 import json
 import binascii
-from typing import Dict, Union
+from typing import Dict, Union, Optional
 from decimal import Decimal
 from collections import OrderedDict
 
@@ -27,6 +27,9 @@ class Msg:
     def to_dict(self) -> Dict:
         return {}
 
+    def to_sign_dict(self) -> Dict:
+        return {}
+
     def to_protobuf(self):
         pass
 
@@ -113,8 +116,9 @@ class NewOrderMsg(Msg):
 
     AMINO_MESSAGE_TYPE = b"CE6DC043"
 
-    def __init__(self, wallet: Wallet, symbol: str, time_in_force: TimeInForce, order_type: OrderType, side: OrderSide,
-                 price: Union[int, float, Decimal], quantity: Union[int, float, Decimal]):
+    def __init__(self, symbol: str, time_in_force: TimeInForce, order_type: OrderType, side: OrderSide,
+                 price: Union[int, float, Decimal], quantity: Union[int, float, Decimal],
+                 wallet: Optional[Wallet] = None):
         """NewOrder transaction creates a new order to buy and sell tokens on Binance DEX.
 
         :param symbol: symbol for trading pair in full name of the tokens e.g. 'ANN-457_BNB'
@@ -127,42 +131,58 @@ class NewOrderMsg(Msg):
         """
         super().__init__(wallet)
         self._symbol = symbol
-        self._time_in_force = NewOrderMsg.TIME_IN_FORCE_INT[time_in_force]
-        self._order_type = NewOrderMsg.ORDER_TYPE_INT[order_type]
-        self._side = NewOrderMsg.ORDER_SIDE_INT[side]
-        self._price = encode_number(price)
-        self._quantity = encode_number(quantity)
+        self._time_in_force = time_in_force
+        self._time_in_force_amino = NewOrderMsg.TIME_IN_FORCE_INT[time_in_force]
+        self._order_type = order_type
+        self._order_type_amino = NewOrderMsg.ORDER_TYPE_INT[order_type]
+        self._side = side
+        self._side_amino = NewOrderMsg.ORDER_SIDE_INT[side]
+        self._price = price
+        self._price_encoded = encode_number(price)
+        self._quantity = quantity
+        self._quantity_encoded = encode_number(quantity)
 
     def to_dict(self) -> Dict:
         return OrderedDict([
             ('id', self._wallet.generate_order_id()),
-            ('ordertype', self._order_type),
-            ('price', self._price),
-            ('quantity', self._quantity),
+            ('ordertype', self._order_type_amino),
+            ('price', self._price_encoded),
+            ('quantity', self._quantity_encoded),
             ('sender', self._wallet.address),
-            ('side', self._side),
+            ('side', self._side_amino),
             ('symbol', self._symbol),
-            ('timeinforce', self._time_in_force),
+            ('timeinforce', self._time_in_force_amino),
         ])
 
+    def to_sign_dict(self) -> Dict:
+        return{
+            'order_type': self._order_type,
+            'price': self._price,
+            'quantity': self._quantity,
+            'side': self._side,
+            'symbol': self._symbol,
+            'time_in_force': self._time_in_force,
+        }
+
     def to_protobuf(self) -> NewOrder:
         pb = NewOrder()
         pb.sender = self._wallet.address_decoded
         pb.id = self._wallet.generate_order_id()
         pb.symbol = self._symbol.encode()
-        pb.timeinforce = self._time_in_force
-        pb.ordertype = self._order_type
-        pb.side = self._side
-        pb.price = self._price
-        pb.quantity = self._quantity
+        pb.timeinforce = self._time_in_force_amino
+        pb.ordertype = self._order_type_amino
+        pb.side = self._side_amino
+        pb.price = self._price_encoded
+        pb.quantity = self._quantity_encoded
         return pb
 
 
 class LimitOrderMsg(NewOrderMsg):
 
-    def __init__(self, wallet: Wallet, symbol: str, side: OrderSide,
+    def __init__(self, symbol: str, side: OrderSide,
                  price: Union[int, float, Decimal], quantity: Union[int, float, Decimal],
-                 time_in_force: TimeInForce = TimeInForce.GOOD_TILL_EXPIRE):
+                 time_in_force: TimeInForce = TimeInForce.GOOD_TILL_EXPIRE,
+                 wallet: Optional[Wallet] = None):
         """NewOrder transaction creates a new order to buy and sell tokens on Binance DEX.
 
         :param symbol: symbol for trading pair in full name of the tokens e.g. 'ANN-457_BNB'
@@ -185,9 +205,9 @@ class LimitOrderMsg(NewOrderMsg):
 
 class LimitOrderBuyMsg(LimitOrderMsg):
 
-    def __init__(self, wallet: Wallet, symbol: str,
-                 price: Union[int, float, Decimal], quantity: Union[int, float, Decimal],
-                 time_in_force: TimeInForce = TimeInForce.GOOD_TILL_EXPIRE):
+    def __init__(self, symbol: str, price: Union[int, float, Decimal], quantity: Union[int, float, Decimal],
+                 time_in_force: TimeInForce = TimeInForce.GOOD_TILL_EXPIRE,
+                 wallet: Optional[Wallet] = None):
         """LimitOrderBuyMsg transaction creates a new limit order buy message on Binance DEX.
 
         :param symbol: symbol for trading pair in full name of the tokens e.g. 'ANN-457_BNB'
@@ -208,9 +228,9 @@ class LimitOrderBuyMsg(LimitOrderMsg):
 
 class LimitOrderSellMsg(LimitOrderMsg):
 
-    def __init__(self, wallet: Wallet, symbol: str,
-                 price: Union[int, float, Decimal], quantity: Union[int, float, Decimal],
-                 time_in_force: TimeInForce = TimeInForce.GOOD_TILL_EXPIRE):
+    def __init__(self, symbol: str, price: Union[int, float, Decimal], quantity: Union[int, float, Decimal],
+                 time_in_force: TimeInForce = TimeInForce.GOOD_TILL_EXPIRE,
+                 wallet: Optional[Wallet] = None):
         """LimitOrderSellMsg transaction creates a new limit order sell message on Binance DEX.
 
         :param symbol: symbol for trading pair in full name of the tokens e.g. 'ANN-457_BNB'
@@ -234,7 +254,7 @@ class CancelOrderMsg(Msg):
 
     AMINO_MESSAGE_TYPE = b"166E681B"
 
-    def __init__(self, wallet: Wallet, symbol: str, order_id: str):
+    def __init__(self, symbol: str, order_id: str, wallet: Optional[Wallet] = None):
         """Cancel transactions cancel the outstanding (unfilled) orders from the Binance DEX. After cancel success,
         the locked quantity on the orders would return back to the address' balance and become free to use,
         i.e. transfer or send new orders.
@@ -254,6 +274,12 @@ class CancelOrderMsg(Msg):
             ('symbol', self._symbol),
         ])
 
+    def to_sign_dict(self) -> Dict:
+        return {
+            'refid': self._order_id,
+            'symbol': self._symbol,
+        }
+
     def to_protobuf(self) -> CancelOrder:
         pb = CancelOrder()
         pb.sender = self._wallet.address_decoded
@@ -266,7 +292,7 @@ class FreezeMsg(Msg):
 
     AMINO_MESSAGE_TYPE = b"E774B32D"
 
-    def __init__(self, wallet: Wallet, symbol: str, amount: Union[int, float, Decimal]):
+    def __init__(self, symbol: str, amount: Union[int, float, Decimal], wallet: Optional[Wallet] = None):
         """Freeze transaction moves the amount of the tokens into a frozen state,
         in which it cannot be used to transfer or send new orders.
 
@@ -275,20 +301,27 @@ class FreezeMsg(Msg):
         """
         super().__init__(wallet)
         self._symbol = symbol
-        self._amount = encode_number(amount)
+        self._amount = amount
+        self._amount_amino = encode_number(amount)
 
     def to_dict(self):
         return OrderedDict([
-            ('amount', self._amount),
+            ('amount', self._amount_amino),
             ('from', self._wallet.address),
             ('symbol', self._symbol),
         ])
 
+    def to_sign_dict(self) -> Dict:
+        return {
+            'amount': self._amount,
+            'symbol': self._symbol,
+        }
+
     def to_protobuf(self) -> TokenFreeze:
         pb = TokenFreeze()
         setattr(pb, 'from', self._wallet.address_decoded)
         pb.symbol = self._symbol.encode()
-        pb.amount = self._amount
+        pb.amount = self._amount_amino
         return pb
 
 
@@ -296,7 +329,7 @@ class UnFreezeMsg(Msg):
 
     AMINO_MESSAGE_TYPE = b"6515FF0D"
 
-    def __init__(self, wallet: Wallet, symbol: str, amount: Union[int, float, Decimal]):
+    def __init__(self, symbol: str, amount: Union[int, float, Decimal], wallet: Optional[Wallet] = None):
         """Turn the amount of frozen tokens back to free state.
 
         :param symbol: token symbol, in full name with "-" suffix
@@ -304,20 +337,27 @@ class UnFreezeMsg(Msg):
         """
         super().__init__(wallet)
         self._symbol = symbol
-        self._amount = encode_number(amount)
+        self._amount = amount
+        self._amount_amino = encode_number(amount)
 
     def to_dict(self):
         return OrderedDict([
-            ('amount', self._amount),
+            ('amount', self._amount_amino),
             ('from', self._wallet.address),
             ('symbol', self._symbol),
         ])
 
+    def to_sign_dict(self) -> Dict:
+        return {
+            'amount': self._amount,
+            'symbol': self._symbol,
+        }
+
     def to_protobuf(self) -> TokenUnfreeze:
         pb = TokenUnfreeze()
         setattr(pb, 'from', self._wallet.address_decoded)
         pb.symbol = self._symbol.encode()
-        pb.amount = self._amount
+        pb.amount = self._amount_amino
         return pb
 
 
@@ -329,7 +369,7 @@ class SignatureMsg(Msg):
         super().__init__(msg.wallet)
         self._signature = Signature(msg)
 
-    def to_protobuf(self) -> NewOrder:
+    def to_protobuf(self) -> StdSignature:
         pub_key_msg = PubKeyMsg(self._wallet)
         std_sig = StdSignature()
         std_sig.sequence = self._wallet.sequence
@@ -353,7 +393,7 @@ class StdTxMsg(Msg):
         self._memo = memo
         self._source = BROADCAST_SOURCE
 
-    def to_protobuf(self) -> NewOrder:
+    def to_protobuf(self) -> StdTx:
         stdtx = StdTx()
         stdtx.msgs.extend([self._msg.to_amino()])
         stdtx.signatures.extend([self._signature.to_amino()])
@@ -389,19 +429,19 @@ class TransferMsg(Msg):
 
     AMINO_MESSAGE_TYPE = b"2A2C87FA"
 
-    def __init__(self, wallet: Wallet, symbol: str, amount: Union[int, float, Decimal],
-                 from_address: str, to_address: str):
+    def __init__(self, symbol: str, amount: Union[int, float, Decimal],
+                 to_address: str, wallet: Optional[Wallet] = None):
         """Transferring funds between different addresses.
 
         :param symbol: token symbol, in full name with "-" suffix
         :param amount: amount of token to freeze
-        :param from_address: amount of token to freeze
         :param to_address: amount of token to freeze
         """
         super().__init__(wallet)
         self._symbol = symbol
-        self._amount = encode_number(amount)
-        self._from_address = from_address
+        self._amount = amount
+        self._amount_amino = encode_number(amount)
+        self._from_address = wallet.address if wallet else None
         self._to_address = to_address
 
     def to_dict(self):
@@ -411,7 +451,7 @@ class TransferMsg(Msg):
                     ('address', self._from_address),
                     ('coins', [
                         OrderedDict([
-                            ('amount', self._amount),
+                            ('amount', self._amount_amino),
                             ('denom', self._symbol)
                         ])
                     ])
@@ -422,7 +462,7 @@ class TransferMsg(Msg):
                     ('address', self._to_address),
                     ('coins', [
                         OrderedDict([
-                            ('amount', self._amount),
+                            ('amount', self._amount_amino),
                             ('denom', self._symbol)
                         ])
                     ])
@@ -430,10 +470,17 @@ class TransferMsg(Msg):
             ])
         ])
 
-    def to_protobuf(self) -> TokenFreeze:
+    def to_sign_dict(self):
+        return {
+            'to_address': self._to_address,
+            'amount': self._amount,
+            'denom': self._symbol,
+        }
+
+    def to_protobuf(self) -> Send:
         token = Token()
         token.denom = self._symbol
-        token.amount = self._amount
+        token.amount = self._amount_amino
         input_addr = Input()
         input_addr.address = decode_address(self._from_address)
         input_addr.coins.extend([token])
